/*
 * File:   main.c
 * Author: alexander.zeller
 *
 * Created on 3. Mai 2018, 14:17
 */


#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "Pic16F1527_hardware.h"
#include "LCD.h"
#include "amb2621.h"
#include "BLE_LED_Control.h"
#include "eusart1.h"
#include "interrupt_manager.h"

#define BAUDRATE_INDEX  AMB2621_BaudRate_19200
#define CONNECTION_TIMING AMB2621_ConnectionTiming_1

void
main(void)
{


    // CONFIG1
#pragma config FOSC = 0b100     // Oscillator Selection Bits 
    //(INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = 0b11      // Watchdog Timer Enable (WDT enabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select 
    //(MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection 
    //(Program memory code protection is disabled)
#pragma config BOREN = 0b00     // Brown-out Reset Enable 
    //(Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable 
    //(CLKOUT function is disabled. I/O or oscillator function on 
    //the CLKOUT pin)

    // CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection 
    //(Write protection off)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable 
    //(Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection 
    //(Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset 
    //(Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable
    //(Low-voltage programming enabled)




    setSystemTimer();
    OscConfig();
    WDT_Config();
    PinConfig();
    initPWM();

    initLCD();

    EUSART1_Initialize((uint8_t) BAUDRATE_INDEX);

    INTERRUPT_GlobalInterruptEnable();
    INTERRUPT_PeripheralInterruptEnable();

    uint16_t oldtime = 0;

    /* used as temp for getting/setting usersettings*/
    uint8_t tempValues[32];
    uint8_t tempLength = 0x00;

    /* used to store rx uart data for further usage*/
    uint8_t pRxBuffer[64];
    uint8_t rxLength = 0x00;
    uint8_t baudrateIndex = 0;


    /* detect baudrate of AMB2621 */
    while(baudrateIndex < 5)
    {
        if (AMB2621_Open())
        {
            break;
        }
        else
        {
            setBaudrate(baudrateIndex++);
        }
    }

    while(1)
    {

        if (0 < EUSART1_is_rx_ready())
        {
            AMB2621_InterpretUartRx();
        }

        switch (ble_state)
        {
            case State_Module_Reset:
            {
                if (AMB2621_PinReset())
                {
                    __delay_ms(4); /* Module resets after this command. Wait to be ready */
                    /* After receiving startup indication, ble_state = State_Uart_OPen*/
                }

                break;
            }

            case State_Uart_Open:
            {
                /* Configure Module for usage */
                /* if already connected to another BT module, disconnect to configure module*/
                AMB2621_BLE_Action_t action;
                AMB2621_BLE_Role_t role;
                if (AMB2621_GetState(&role, &action, &tempValues[1], &tempLength))
                {
                    if (AMB2621_BLE_Action_Connected == action)
                    {
                        /* will set ble_state to disconnect on scuccess */
                        AMB2621_Disconnect();
                    }
                }

                /* Connection timing to 2 to communcate with apple device*/
                if (AMB2621_GetConnectionTiming(&tempValues[0]))
                {
                    if (CONNECTION_TIMING != tempValues[0])
                    {
                        AMB2621_SetConnectionTiming(CONNECTION_TIMING);
                        __delay_ms(4); /* Module resets after this command. Wait to be ready */
                    }
                }

                /* Set SecFlag*/
                if (AMB2621_GetSecFlag(&tempValues[0]))
                {
                    if (AMB2621_SecFalgs_NONE != tempValues[0])
                    {
                        AMB2621_SetSecFlag(AMB2621_SecFalgs_NONE);
                        __delay_ms(4); /* Module resets after this command. Wait to be ready */
                    }
                }

                if (AMB2621_GetBaudrateIndex(&tempValues[0]))
                {
                    if (BAUDRATE_INDEX != tempValues[0])
                    {
                        if (AMB2621_SetBaudrateIndex(BAUDRATE_INDEX))
                        {
                            setBaudrate(BAUDRATE_INDEX);
                        }
                    }
                }

                ble_state = State_BLE_Disconnected;

                break;
            }

            case State_BLE_Channel_Open:
            {
                uint8_t maxPayload = (uint8_t) 0;
                AMB2621_GetBLEMaxPayloadLength(&maxPayload);

                //other ble device has not enough payload capaticity for the protocol used to control led channels via bt */
                if (maxPayload < (uint8_t) 25)
                {
                    AMB2621_Disconnect();
                }
                else
                {
                    /* update app */
                    BLE_LED_control_InitChannels();
                    BLE_LED_control_GetChannelValuesAsPacket(tempValues, &tempLength);
                    //BLE_LED_control_BleToPWM(tempValues, tempLength);
                    AMB2621_Transmit(tempValues, tempLength);
                    ble_state = State_BLE_Wait_For_Data;

                    /* update */
                }
                break;
            }

            case State_BLE_Wait_For_Data:
            {
                /* Wait for BLE Data */
                __delay_ms(1);
                break;
            }

            case State_BLE_Data_Rcvd:
            {
                if (AMB2621_GetBLEData(pRxBuffer, &rxLength))
                {
                    BLE_LED_control_BleToPWM(&pRxBuffer[7], (uint8_t) (rxLength - 7));
                    BLE_LED_control_GetChannelValuesAsPacket(tempValues, &tempLength);
                    AMB2621_Transmit(tempValues, tempLength);
                    pwmToString();
                    putStrLCD2();
                }
                ble_state = State_BLE_Wait_For_Data;
                break;
            }

                /* Uart communication with module open, but no ble device connected*/
            case State_BLE_Disconnected:
            {
                pwmToString();
                SwitchOrPWM();
                putStrLCD();
                /* Wait until 1ms is over for next loop */
                while(oldtime == getSystemTimeMillis())
                {
                }
                oldtime = getSystemTimeMillis();
            }
        }
    }
    return;

};
