/**
 * \file sourcecode.c
 * \brief
 *
 * \since
 * \author
 */

/* ###########################################################################
 * Includes:
 */

#include <xc.h>
#include <string.h>
#include <stdint.h>
#include <stdbool.h>
#include "eusart1.h"
#include "amb2621.h"


/*
 * Includes.
 * ######################################################################### */

/* ###########################################################################
 * Macros & Typedefs:
 */

/* #############################
 * Constant Macros:
 */
#define _XTAL_FREQ  16000000        // this is used by the __delay_ms(xx) and __delay_us(xx) functions

#define MAX_PAYLOAD_LENGTH              (uint16_t)48
#define LENGTH_CMD_OVERHEAD             (uint16_t)5
#define LENGTH_CMD_OVERHEAD_WITHOUT_CRC (uint16_t)(LENGTH_CMD_OVERHEAD -1)
#define MAX_CMD_LENGTH                  (uint16_t)(MAX_PAYLOAD_LENGTH + LENGTH_CMD_OVERHEAD)
#define CMD_WAIT_TIME                   (uint16_t)500
#define UART_RETRIES                    (uint8_t)5
#define TIME_STEP_MS                    (uint16_t)3

#define CMD_POSITION_STX        (uint8_t)0
#define CMD_POSITION_CMD        (uint8_t)1
#define CMD_POSITION_LENGTH_LSB (uint8_t)2
#define CMD_POSITION_LENGTH_MSB (uint8_t)3
#define CMD_POSITION_DATA       (uint8_t)4
/*
 * Constant Macros.
 * ########################### */

/* #############################
 * Function Macros:
 */

#define CMD_ARRAY_SIZE() ((((uint16_t)CMD_Array[CMD_POSITION_LENGTH_LSB] << 0) | ((uint16_t)CMD_Array[CMD_POSITION_LENGTH_MSB] << 8)) + LENGTH_CMD_OVERHEAD)
/*
 * Function Macros.
 * ########################## */

/* #############################
 * Typedefs:
 */

/* type used to check the response, when a command was sent to the AMB8826 */
typedef enum CMD_Status_t
{
    CMD_Status_Success = (uint8_t) 0x00,
    CMD_Status_Failed,
    CMD_Status_Invalid,
    CMD_Status_Reset,
    CMD_Status_NoStatus,
} CMD_Status_t;

typedef struct
{
    uint8_t cmd; /* variable to check if correct CMD has been confirmed */
    CMD_Status_t status; /* variable used to check the response (*_CNF), when a request (*_REQ) was sent to the AMB8826 */
} CMD_Confirmation_t;

/*
 * Typedefs.
 * ########################## */

/*
 * Macros & Typedefs.
 * ######################################################################## */

/* ##########################################################################
 * Declarations:
 */

/* #############################
 * Exported Globals:
 */


/*
 * Exported Globals.
 * ########################## */

/* #############################
 * Static Globals:
 */
static volatile uint8_t rc1_error_state;
static uint8_t CMD_Array[MAX_CMD_LENGTH]; /* for UART TX to module*/

static uint8_t rxBuffer[MAX_CMD_LENGTH]; /* For UART RX from module */
static bool rxTimeout;

static bool askedForState;

CMD_Confirmation_t cmdConfirmation;
static uint8_t maxPayloadBLEConnection;

/*
 * Static Globals.
 * ########################## */

/* #############################
 * Static Functions:
 */

static void init();
static bool FillChecksum(uint8_t* pArray, uint16_t length);
static void SendBytes(uint8_t *pDataToSend, uint16_t length);
static void HandleRxPacket(uint8_t *pRxBuffer);
static bool Wait4CNF(uint16_t max_time_ms, uint8_t expectedCmdConfirmation, CMD_Status_t expectedStatus, bool reset_confirmstate);

/*
 * Static Functions.
 * ########################## */

/*
 * Declarations.
 * ######################################################################## */

/* ###########################################################################
 * Definitions:
 */

/* #############################
 * Exported Functions:
 */


bool
AMB2621_Open()
{
    init();


    // AMBER Pin configuration

    // Initialite WakeUp Pin to output high
    TRISCbits.TRISC0 = 0;
    PORTCbits.RC0 = 1;

    // Initialize Reset Pin to output high
    TRISEbits.TRISE1 = 0; // Output
    ANSELEbits.ANSE1 = 0; //Digital
    PORTEbits.RE1 = 1; // Reset amb2621 high

    /* empty rx rx uart buffer before pinreset*/
    flushRxBuffer();

    return AMB2621_PinReset();
}

bool
AMB2621_InterpretUartRx()
{
    uint16_t BytesToReceive = (uint16_t) 0;
    uint16_t rxByteCounter = (uint16_t) 0;
    uint8_t rxChecksum = (uint8_t) 0;
    uint8_t rcvdByte = (uint8_t) 0;

    bool waitForPacket = true;
    uint8_t retryCounter = (uint8_t) 0;

    rxTimeout = false;
    startUartTimer();
    
    while(waitForPacket)
    {

        if (rxTimeout)
        {
            /* Startindication (0x02) was received, but rx of complete package took too long */
            rxTimeout = false;
            return false;
        }

        /* EUSART1_Read is blocking. So make sure data is really available before the next read*/
        //        retryCounter = (uint8_t)0;
        //        while((uint8_t)0 == EUSART1_is_rx_ready())
        //        {
        //            if (retryCounter >= UART_RETRIES)
        //            {
        //                return false;
        //            }
        //
        //            /* wait */
        //            retryCounter++;
        //            __delay_ms(TIME_STEP_MS);
        //        }

        if (0 < EUSART1_is_rx_ready())
        {
            rcvdByte = EUSART1_Read();
            switch (rxByteCounter)
            {
                    /* CMD_STX*/
                case(0):
                {
                    if (CMD_STX == rcvdByte)
                    {
                        stopUartTimer();
                        startUartTimer();
                        
                        rxBuffer[rxByteCounter] = rcvdByte;
                        rxChecksum = rcvdByte;
                        rxByteCounter = (uint8_t) 1;
                    }
                    // else ignore
                    break;
                }
                    /* CMD*/
                case(1):
                {
                    rxBuffer[rxByteCounter] = rcvdByte;
                    rxChecksum ^= rcvdByte;
                    rxByteCounter++;

                    break;
                }
                    /* Length LSB*/
                case(2):
                {
                    rxBuffer[rxByteCounter] = rcvdByte;
                    BytesToReceive = (uint16_t) (rcvdByte << 0);
                    rxChecksum ^= rcvdByte;
                    rxByteCounter++;

                    break;
                }
                    /* Length MSB*/
                case(3):
                {
                    rxBuffer[rxByteCounter] = rcvdByte;

                    BytesToReceive += (uint16_t) (rcvdByte << 8) + LENGTH_CMD_OVERHEAD;
                    rxChecksum ^= rcvdByte;
                    rxByteCounter++;

                    break;
                }
                    /* payload + crc */
                default:
                {
                    rxBuffer[rxByteCounter] = rcvdByte;
                    rxChecksum ^= rcvdByte;
                    rxByteCounter++;

                    if (rxByteCounter == BytesToReceive)
                    {
                        stopUartTimer();
                        if ((uint8_t) 0 == rxChecksum)
                        {
                            // All good, do Stuff

                            HandleRxPacket(&rxBuffer[0]);
                            rxByteCounter = (uint16_t) 0;
                            BytesToReceive = (uint16_t) 0;
                            waitForPacket = false;
                        }
                        else
                        {
                            /* restart */
                            rxByteCounter = (uint16_t) 0;
                        }

                    }

                    break;
                }
            }
        }
    }

    return true;
}

bool
AMB2621_PinReset()
{

    ble_state = State_Module_Reset;

    PORTEbits.RE1 = 0; // Reset amb2621 low
    __delay_ms(5);
    rc1_error_state = RC1STA;
    PORTEbits.RE1 = 1; // Reset amb2621 high

    return Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_GETSTATE_CNF, CMD_Status_NoStatus, true);
}

bool
AMB2621_PinWakeup()
{
    ble_state = State_Module_Reset;

    PORTCbits.RC0 = 0; // wakup amb2621 low
    __delay_ms(5);
    rc1_error_state = RC1STA;
    PORTCbits.RC0 = 1; // wakeup amb2621 high

    return Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_GETSTATE_CNF, CMD_Status_NoStatus, true);
}

bool
AMB2621_Reset()
{
    bool ret = false;

    /* fill CMD_ARRAY packet */
    CMD_Array[CMD_POSITION_STX] = CMD_STX;
    CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_RESET_REQ;
    CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) 0;
    CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) 0;

    if (FillChecksum(CMD_Array, CMD_ARRAY_SIZE()))
    {
        /* now send CMD_ARRAY */
        SendBytes(CMD_Array, CMD_ARRAY_SIZE());

        ble_state = State_Module_Reset;
        /* wait for cnf */
        ret = Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_RESET_CNF, CMD_Status_Success, true);
    }
    return ret;
}

bool
AMB2621_Disconnect()
{
    bool ret = false;
    /* fill CMD_ARRAY packet */
    CMD_Array[CMD_POSITION_STX] = CMD_STX;
    CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_DISCONNECT_REQ;
    CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) 0;
    CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) 0;

    if (FillChecksum(CMD_Array, CMD_ARRAY_SIZE()))
    {
        /* now send CMD_ARRAY */
        SendBytes(CMD_Array, CMD_ARRAY_SIZE());

        /* wait for cnf */
        ret = Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_DISCONNECT_CNF, CMD_Status_Success, true);
    }
    return ret;
}

bool
AMB2621_Sleep()
{
    bool ret = false;
    /* fill CMD_ARRAY packet */
    CMD_Array[CMD_POSITION_STX] = CMD_STX;
    CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_SLEEP_REQ;
    CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) 0;
    CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) 0;

    if (FillChecksum(CMD_Array, CMD_ARRAY_SIZE()))
    {
        /* now send CMD_ARRAY */
        SendBytes(CMD_Array, CMD_ARRAY_SIZE());

        /* wait for cnf */
        ret = Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_SLEEP_CNF, CMD_Status_Success, true);
    }
    return ret;
}

bool
AMB2621_Transmit(uint8_t *pPayload, uint16_t length)
{
    bool ret = false;
    if (length < MAX_PAYLOAD_LENGTH)
    {
        CMD_Array[CMD_POSITION_STX] = CMD_STX;
        CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_DATA_REQ;
        CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) (length >> 0);
        CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) (length >> 8);

        memcpy(&CMD_Array[CMD_POSITION_DATA], pPayload, length);

        if (FillChecksum(&CMD_Array[0], (length + LENGTH_CMD_OVERHEAD)))
        {
            SendBytes(&CMD_Array[0], (length + LENGTH_CMD_OVERHEAD));
            if (Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_GETSTATE_CNF, CMD_Status_Failed, true))
            {
                ret = true;
            }
        }
    }
    else
    {
        return ret;
    }

    return ret;
}

extern void
AMB2621_GetBLEMaxPayloadLength(uint8_t *maxPayload)
{
    *maxPayload = maxPayloadBLEConnection;
}

bool
AMB2621_GetBLEData(uint8_t *pBuffer, uint8_t *pLength)
{
    bool ret = false;

    if (ble_state == State_BLE_Data_Rcvd)
    {
        *pLength = ((((uint16_t) rxBuffer[CMD_POSITION_LENGTH_LSB] << 0) | ((uint16_t) rxBuffer[CMD_POSITION_LENGTH_MSB] << 8)));
        memcpy(pBuffer, &rxBuffer[CMD_POSITION_DATA], *pLength);

        ret = true;
    }

    return ret;
}

bool
AMB2621_WaitForStartupIndication(uint16_t max_time_ms)
{
    uint16_t count = 0;
    uint16_t max_count = max_time_ms / (TIME_STEP_MS * UART_RETRIES);

    while(State_Uart_Open != ble_state)
    {
        if (count <= max_count)
        {
            AMB2621_InterpretUartRx();
            count++;
        }
        else
        {
            /* no channel open within timeout*/
            return false;
        }

    }

    return true;
}

bool
AMB2621_FactoryReset()
{
    bool ret = false;
    /* fill CMD_ARRAY packet */
    CMD_Array[CMD_POSITION_STX] = CMD_STX;
    CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_FACTORYRESET_REQ;
    CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) 0;
    CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) 0;

    if (FillChecksum(CMD_Array, CMD_ARRAY_SIZE()))
    {
        /* now send CMD_ARRAY */
        SendBytes(CMD_Array, CMD_ARRAY_SIZE());

        /* wait for cnf */
        ret = Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_FACTORYRESET_CNF, CMD_Status_Success, true);
    }
    return ret;

}

bool
AMB2621_Set(uint8_t userSetting, uint8_t *pValue, uint8_t length)
{
    bool ret = false;

    /* fill CMD_ARRAY packet */
    CMD_Array[CMD_POSITION_STX] = CMD_STX;
    CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_SET_REQ;
    CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) (1 + length);
    CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) 0;
    CMD_Array[CMD_POSITION_DATA] = userSetting;
    memcpy(&CMD_Array[CMD_POSITION_DATA + 1], pValue, length);

    if (FillChecksum(CMD_Array, CMD_ARRAY_SIZE()))
    {
        /* now send CMD_ARRAY */
        SendBytes(CMD_Array, CMD_ARRAY_SIZE());

        /* wait for cnf */
        ret = Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_SET_CNF, CMD_Status_Success, true);
    }
    return ret;
}

bool
AMB2621_SetDeviceName(uint8_t *deviceName, uint8_t nameLength)
{
    return AMB2621_Set(AMB2621_USERSETTING_POSITION_RF_DEVICE_NAME, deviceName, nameLength);
}

bool
AMB2621_SetAdvertisingTimeout(uint16_t advTimeout)
{

    return AMB2621_Set(AMB2621_USERSETTING_POSITION_RF_ADVERTISING_TIMEOUT, (uint8_t*) & advTimeout, 2);
}

bool
AMB2621_SetConnectionTiming(uint8_t connectionTiming)
{
    bool ret = false;
    if (connectionTiming < (uint8_t) 7)
    {
        ret = AMB2621_Set(AMB2621_USERSETTING_POSITION_RF_CONNECTION_TIMING, &connectionTiming, 1);
    }

    return ret;
}

bool
AMB2621_SetSecFlag(AMB2621_SecMode_t secMode)
{
    uint8_t value = secMode;
    return AMB2621_Set(AMB2621_USERSETTING_POSITION_RF_SEC_FLAGS, &value, 1);
}

bool
AMB2621_SetBaudrateIndex(AMB2621_BaudRate_t baudrateIndex)
{
    uint8_t value = baudrateIndex;
    return AMB2621_Set(AMB2621_USERSETTING_POSITION_UART_BAUDRATE_INDEX, &value, 1);
}

bool
AMB2621_SetStaticPasskey(uint8_t *staticPasskey)
{
    return AMB2621_Set(AMB2621_USERSETTING_POSITION_RF_STATIC_PASSKEY, staticPasskey, 6);
}

bool
AMB2621_Get(uint8_t userSetting, uint8_t *pResponse, uint8_t *pResponse_Length)
{
    bool ret = false;

    /* fill CMD_ARRAY packet */
    CMD_Array[CMD_POSITION_STX] = CMD_STX;
    CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_GET_REQ;
    CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) 1;
    CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) 0;
    CMD_Array[CMD_POSITION_DATA] = userSetting;

    if (FillChecksum(CMD_Array, CMD_ARRAY_SIZE()))
    {
        /* now send CMD_ARRAY */
        SendBytes(CMD_Array, CMD_ARRAY_SIZE());

        /* wait for cnf */
        if (Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_GET_CNF, CMD_Status_Success, true))
        {
            uint16_t length = ((uint16_t) rxBuffer[CMD_POSITION_LENGTH_LSB] << 0) + ((uint16_t) rxBuffer[CMD_POSITION_LENGTH_MSB] << 8);
            memcpy(pResponse, &rxBuffer[CMD_POSITION_DATA + 1], length - 1); /* First Data byte is status, following bytes response*/
            *pResponse_Length = length - 1;
            ret = true;
        }
    }
    return ret;
}

bool
AMB2621_GetDeviceName(uint8_t *deviceName, uint8_t *nameLength)
{
    return AMB2621_Get(AMB2621_USERSETTING_POSITION_FS_DEVICE_INFO, deviceName, nameLength);
}

bool
AMB2621_GetMAC(uint8_t *pMac)
{
    uint8_t length;
    return AMB2621_Get(AMB2621_USERSETTING_POSITION_FS_MAC, pMac, &length);
}

bool
AMB2621_GetBTMAC(uint8_t *pMac)
{
    uint8_t length;
    return AMB2621_Get(AMB2621_USERSETTING_POSITION_FS_BTMAC, pMac, &length);
}

bool
AMB2621_GetAdvertisingTimeout(uint8_t *advTimeout)
{
    uint8_t length;
    return AMB2621_Get(AMB2621_USERSETTING_POSITION_RF_ADVERTISING_TIMEOUT, advTimeout, &length);
}

bool
AMB2621_GetConnectionTiming(uint8_t *connectionTiming)
{
    uint8_t length;
    return AMB2621_Get(AMB2621_USERSETTING_POSITION_RF_CONNECTION_TIMING, connectionTiming, &length);
}

bool
AMB2621_GetSecFlag(uint8_t *psecMode)
{
    uint8_t length;
    return AMB2621_Get(AMB2621_USERSETTING_POSITION_RF_SEC_FLAGS, psecMode, &length);
}

bool
AMB2621_GetBaudrateIndex(uint8_t *baudrateIndex)
{
    uint8_t length;
    return AMB2621_Get(AMB2621_USERSETTING_POSITION_UART_BAUDRATE_INDEX, baudrateIndex, &length);
}

bool
AMB2621_GetState(AMB2621_BLE_Role_t *pBLE_role, AMB2621_BLE_Action_t *pBLE_action, uint8_t *pInfo, uint8_t *pLength)
{
    bool ret = false;

    /* fill CMD_ARRAY packet */
    CMD_Array[CMD_POSITION_STX] = CMD_STX;
    CMD_Array[CMD_POSITION_CMD] = AMB2621_CMD_GETSTATE_REQ;
    CMD_Array[CMD_POSITION_LENGTH_LSB] = (uint8_t) 0;
    CMD_Array[CMD_POSITION_LENGTH_MSB] = (uint8_t) 0;

    if (FillChecksum(CMD_Array, CMD_ARRAY_SIZE()))
    {
        /* now send CMD_ARRAY */
        SendBytes(CMD_Array, CMD_ARRAY_SIZE());
        askedForState = true;
        /* wait for cnf */
        if (Wait4CNF(CMD_WAIT_TIME, AMB2621_CMD_GETSTATE_CNF, CMD_Status_NoStatus, true))
        {
            uint16_t length = ((uint16_t) rxBuffer[CMD_POSITION_LENGTH_LSB] << 0) + ((uint16_t) rxBuffer[CMD_POSITION_LENGTH_MSB] << 8);
            *pBLE_role = rxBuffer[CMD_POSITION_DATA];
            *pBLE_action = rxBuffer[CMD_POSITION_DATA + 1];
            memcpy(pInfo, &rxBuffer[CMD_POSITION_DATA + 2], length - 2);

            *pLength = length;
            ret = true;
        }
    }
    askedForState = false;
    return ret;
}

void
Uart_Timer_Callback()
{
    PIR3bits.TMR4IF = 0;
    PIE3bits.TMR4IE = 0;
    T4CONbits.TMR4ON = 0;
    rxTimeout = true;
}

/**@brief This function does something
 *
 * @param[in]   test1    variable 1
 * @param[in]   test2    variable 2
 * @param[out]  test3P   variable 3
 *
 * @retval      true if test1 is larger than test2. Otherwise false
 */

/*
 * Exported Functions.
 * ########################## */

/* #############################
 * Static Functions:
 */

static void
init()
{
    // Initialize ble_state. Uart_closed assumed until first message is received from module
    ble_state = State_Module_Reset;

    maxPayloadBLEConnection = (uint8_t) 0;
    askedForState = false;
    rxTimeout = false;
    rc1_error_state = (uint8_t) 0;
}

static void
HandleRxPacket(uint8_t * pRxBuffer)
{
    switch (pRxBuffer[CMD_POSITION_CMD])
    {
        case AMB2621_CMD_RESET_CNF:
        {
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = pRxBuffer[CMD_POSITION_DATA];
            break;
        }

        case AMB2621_CMD_GET_CNF:
        {
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = pRxBuffer[CMD_POSITION_DATA];
            break;
        }

        case AMB2621_CMD_SET_CNF:
        {
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = pRxBuffer[CMD_POSITION_DATA];
            break;
        }

        case AMB2621_CMD_GETSTATE_CNF:
        {
            /* reaction to CMD_GETSTATE_REQ*/
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = CMD_Status_NoStatus;

            if (!askedForState)
            {
                /* Startup indication*/
                ble_state = State_Uart_Open;
            }

            break;
        }

        case AMB2621_CMD_CONNECT_CNF:
        {
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = pRxBuffer[CMD_POSITION_DATA];

            break;
        }

        case AMB2621_CMD_DATA_CNF:
        {
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = pRxBuffer[CMD_POSITION_DATA];

            break;
        }

        case AMB2621_CMD_DISCONNECT_CNF:
        {
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = pRxBuffer[CMD_POSITION_DATA];
            break;
        }

        case AMB2621_CMD_FACTORYRESET_CNF:
        {
            cmdConfirmation.cmd = pRxBuffer[CMD_POSITION_CMD];
            cmdConfirmation.status = pRxBuffer[CMD_POSITION_DATA];
            break;
        }

        case AMB2621_CMD_CHANNELOPEN_RSP:
        {
            /* Payload of CHANNELOPEN_RSP: Status (1 byte), BTMACH (6 byte), Max Payload (1byte)*/
            maxPayloadBLEConnection = pRxBuffer[CMD_POSITION_DATA + 7];
            ble_state = State_BLE_Channel_Open;
            break;
        }

        case AMB2621_CMD_RESET_IND:
        {
            ble_state = State_Module_Reset;
            break;
        }


        case AMB2621_CMD_CONNECT_IND:
        {
            break;
        }

        case AMB2621_CMD_DISCONNECT_IND:
        {
            maxPayloadBLEConnection = (uint8_t) 0;
            ble_state = State_BLE_Disconnected;
            break;
        }

        case AMB2621_CMD_DATA_IND:
        {
            ble_state = State_BLE_Data_Rcvd;
            break;
        }

        default:
        {
            /* invalid*/
            break;
        }
    }
}

/* function to add the checksum at the end of the data packet */
static bool
FillChecksum(uint8_t* pArray, uint16_t length)
{
    bool ret = false;

    if ((length >= LENGTH_CMD_OVERHEAD) && (pArray[0] == CMD_STX))
    {
        uint8_t checksum = (uint8_t) 0;
        uint16_t payload_length = (uint16_t) (pArray[3] << 8) + pArray[2];
        uint16_t i = 0;
        for (i = 0;
            i < (payload_length + LENGTH_CMD_OVERHEAD_WITHOUT_CRC);
            i++)
        {
            checksum ^= pArray[i];
        }
        pArray[payload_length + LENGTH_CMD_OVERHEAD_WITHOUT_CRC] = checksum;
        ret = true;
    }
    return ret;
}

static void
SendBytes(uint8_t *pDataToSend, uint16_t length)
{
    uint16_t i;
    for (i = 0; i < length; i++)
    {
        EUSART1_Write(pDataToSend[i]);
    }
}

/* function that waits for the return value of AMB8826 (*_CNF), when a command (*_REQ) was sent before */
static bool
Wait4CNF(uint16_t max_time_ms, uint8_t expectedCmdConfirmation, CMD_Status_t expectedStatus, bool reset_confirmstate)
{
    uint16_t count = (uint16_t) 0;
//    uint16_t max_count = max_time_ms / (TIME_STEP_MS * UART_RETRIES);

    if (reset_confirmstate)
    {
        cmdConfirmation.cmd = 0xFF;
    }
    while(1)
    {
        if (expectedCmdConfirmation == cmdConfirmation.cmd)
        {
            return(bool) (cmdConfirmation.status == expectedStatus);

        }

        if (count >= UART_RETRIES)
        {
            /* received no correct response within timeout */
            return false;
        }

        /* wait */
        count++;
        AMB2621_InterpretUartRx();
    }
    return true;
}


/*
 * Static Functions.
 * ########################## */

/*
 * Definitions.
 * ######################################################################### */